/*
 * Created on 06-nov-2006
 *
 * gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: RouteMemoryDriver.java,v 1.2 2006/11/08 19:32:36 azabala Exp $
 * $Log: RouteMemoryDriver.java,v $
 * Revision 1.2  2006/11/08 19:32:36  azabala
 * *** empty log message ***
 *
 * Revision 1.1  2006/11/07 19:49:28  azabala
 * *** empty log message ***
 *
 *
 */
package com.iver.cit.gvsig.graph.solvers;

import java.awt.geom.Rectangle2D;
import java.io.IOException;
import java.sql.Types;
import java.util.List;

import javax.swing.table.DefaultTableModel;

import com.hardcode.gdbms.engine.data.DataSourceFactory;
import com.hardcode.gdbms.engine.data.driver.DriverException;
import com.hardcode.gdbms.engine.data.edition.DataWare;
import com.hardcode.gdbms.engine.values.Value;
import com.iver.cit.gvsig.fmap.core.FShape;
import com.iver.cit.gvsig.fmap.core.IFeature;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.drivers.DriverAttributes;
import com.iver.cit.gvsig.fmap.drivers.FieldDescription;
import com.iver.cit.gvsig.fmap.drivers.MemoryDriver;
import com.iver.cit.gvsig.fmap.drivers.VectorialDriver;

/**
 * Memory driver to work with Network's Route features
 * 
 * @author azabala
 * 
 */
public class RouteMemoryDriver extends MemoryDriver implements VectorialDriver {
	// TODO Internationalize field names
	static FieldDescription[] fields = new FieldDescription[4];
	static {
		FieldDescription fieldDesc = new FieldDescription();
		fieldDesc.setFieldName("IDARC");
		fieldDesc.setFieldType(Types.INTEGER);
		fieldDesc.setFieldLength(20);
		fieldDesc.setFieldDecimalCount(0);
		fields[0] = fieldDesc;

		fieldDesc = new FieldDescription();
		fieldDesc.setFieldName("WEIGHT");
		fieldDesc.setFieldType(Types.DOUBLE);
		fieldDesc.setFieldLength(20);
		fieldDesc.setFieldDecimalCount(5);
		fields[1] = fieldDesc;

		fieldDesc = new FieldDescription();
		fieldDesc.setFieldName("LENGTH");
		fieldDesc.setFieldType(Types.DOUBLE);
		fieldDesc.setFieldLength(20);
		fieldDesc.setFieldDecimalCount(5);
		fields[2] = fieldDesc;

		fieldDesc = new FieldDescription();
		fieldDesc.setFieldName("TEXT");
		fieldDesc.setFieldType(Types.VARCHAR);
		fieldDesc.setFieldLength(254);
		fieldDesc.setFieldDecimalCount(0);
		fields[3] = fieldDesc;
	}

	
	List features;

	Rectangle2D fullExtent;

	public RouteMemoryDriver(List features) {
		this.features = features;
		computeFullExtent();
	}

	void computeFullExtent() {
		if (features.size() > 0) {
			IFeature feature = (IFeature) features.get(0);
			fullExtent = feature.getGeometry().getBounds2D();
			for (int i = 1; i < features.size(); i++) {
				feature = (IFeature) features.get(i);
				Rectangle2D rAux = feature.getGeometry().getBounds2D();
				fullExtent.add(rAux);
			}
		}
	}

	public int getShapeType() {
		return FShape.LINE;
	}

	public String getName() {
		return "";
	}

	public DriverAttributes getDriverAttributes() {
		return null;
	}

	public boolean isWritable() {
		return false;
	}

	public int[] getPrimaryKeys()
			throws com.hardcode.gdbms.engine.data.driver.DriverException {
		return null;
	}

	public void write(DataWare dataWare)
			throws com.hardcode.gdbms.engine.data.driver.DriverException {
	}

	/**
	 * Returns de field type of the specified field index.
	 * 
	 * @return field type of i field
	 */
	public int getFieldType(int i) throws DriverException {
		// azabala: we overwrite MemoryDriver because type resolution
		// is based in first register value (it could be null)
		if (i >= fields.length)
			throw new DriverException("Excedido el numero de campos");
		return fields[i].getFieldType();
	}

	public int getShapeCount() throws IOException {
		return features.size();
	}

	public Rectangle2D getFullExtent() throws IOException {
		return fullExtent;
	}

	public IGeometry getShape(int index) {
		if(index <  features.size())
			return ((IFeature) features.get(index)).getGeometry();
		else
			return null;
	}

	public void reload() throws IOException, DriverException {

	}
	
	/**
	 * Devuelve el modelo de la tabla.
	 *
	 * @return modelo de la tabla.
	 */
	public DefaultTableModel getTableModel() {
		return null;
	}

	/**
	 * Aade un shape.
	 *
	 * @param geom shape.
	 * @param row fila.
	 */
	public void addGeometry(IGeometry geom, Object[] row) {
	}

	
	/**
	 * Mtodo de conveniencia, para poder aadir directamente un shape
	 * o una IGeometry. (Arriba est el de aadir una IGeometry.
	 * @param shp
	 * @param row
	 */
	public void addShape(FShape shp, Object[] row) {
	}

	/**
	 * Devuelve el extent a partir de un ndice.
	 *
	 * @param index ndice.
	 *
	 * @return Extent.
	 *
	 * @throws IOException
	 */
	public Rectangle2D getShapeBounds(int index) throws IOException {
		return ((IFeature)features.get(index)).getGeometry().getBounds2D();
	}

	
	public int getShapeType(int index) {
		return ((IFeature)features.get(index)).getGeometry().getGeometryType();
	}


	/* (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.ReadDriver#getFieldValue(long, int)
	 */
	public Value getFieldValue(long rowIndex, int fieldId)
		throws DriverException {
		IFeature feature = (IFeature) features.get((int) rowIndex);
		return feature.getAttributes()[fieldId];
	}

	
	public int getFieldCount() throws DriverException {
		return fields.length;
	}

	
	public String getFieldName(int fieldId) throws DriverException {
		return fields[fieldId].getFieldName();
	}

	/* (non-Javadoc)
	 * @see com.hardcode.gdbms.engine.data.ReadDriver#getRowCount()
	 */
	public long getRowCount() throws DriverException {
		return features.size();
	}
    
    /* (non-Javadoc)
     * @see com.hardcode.gdbms.engine.data.driver.GDBMSDriver#setDataSourceFactory(com.hardcode.gdbms.engine.data.DataSourceFactory)
     */
    public void setDataSourceFactory(DataSourceFactory dsf) {
    }

    public int getFieldWidth(int fieldId)
    {
    	// TODO
    	return 30;
    }
}
